// Generated by CoffeeScript 1.3.3
(function() {
  var Connection, NS, async, config, defaultConfigFile, fs, inspect, path, version, xmpp,
    __slice = [].slice;

  fs = require('fs');

  path = require('path');

  async = require('async');

  inspect = require('util').inspect;

  Connection = require('./xmpp/connection');

  xmpp = require('node-xmpp');

  NS = require('./xmpp/ns');

  config = require('jsconfig');

  version = '0.3.5'

  defaultConfigFile = path.join(__dirname, "config.js");

  if (fs.existsSync(defaultConfigFile)) {
    config.defaults(defaultConfigFile);
  }

  process.title = "buddycloud-server " + version;

  config.set('ignore unknown', true);

  config.set('env', {
    HOST: 'xmpp.host',
    PORT: ['xmpp.port', parseInt]
  });

  config.cli({
    host: ['xmpp.host', ['b', "xmpp server listen address", 'host']],
    port: ['xmpp.port', ['p', "xmpp server listen port", 'number']],
    config: ['c', "load config file", 'path'],
    debug: [false, "enable debug mode"],
    nobuild: [false, "[INTERNAL] disable build"],
    stdout: ['logging.stdout', [false, "Log to stdout"]],
    version: [false, "Display version"]
  });

  config.load(function(args, opts) {
    var componentConfig, connection, errors, index, key, logger, logger_, model, pubsubBackend, pubsubServer, router, value, xmppConn, _ref, _ref1, _results,
      _this = this;
    if (opts.version) {
      console.log(version);
      process.exit(0);
    }
    if ((_ref = opts.config) != null ? _ref.length : void 0) {
      if (opts.config[0] !== '/') {
        opts.config = path.join(process.cwd(), opts.config);
      }
      config.merge(opts.config);
    } else if (fs.existsSync("/etc/buddycloud-server/config.js")) {
      config.merge("/etc/buddycloud-server/config.js");
    }
    if (opts.stdout) {
      config.logging.stdout = true;
    }
    logger_ = require('./logger');
    logger_.setConfig(config.logging);
    logger = logger_.makeLogger('main');
    if (opts.debug) {
      process.on('uncaughtException', function(err) {
        return logger.error("uncaughtException: " + (err.stack || err.message || err.toString()));
      });
    }
    errors = require('./errors');
    model = require('./local/model_postgres');
    model.start(config.modelConfig);
    router = new (require('./router').Router)(model, config.checkCreateNode, config.autosubscribeNewUsers);
    xmppConn = new Connection.Connection(config.xmpp);
    pubsubServer = new (require('./xmpp/pubsub_server').PubsubServer)(xmppConn);
    pubsubBackend = new (require('./xmpp/backend_pubsub').PubsubBackend)(xmppConn);
    router.addBackend(pubsubBackend);
    pubsubServer.on('request', function(request) {
      logger.trace("request: " + (inspect(request)));
      if (request.operation === 'get-version') {
        return request.callback(null, {
          name: "buddycloud-server",
          version: version,
          os: process.platform
        });
      } else if (request.sender !== request.actor) {
        return pubsubBackend.authorizeFor(request.sender, request.actor, function(err, valid) {
          if (err) {
            return request.callback(err);
          } else if (!valid) {
            return request.callback(new errors.BadRequest('Requesting service not authorized for actor'));
          } else {
            return router.run(request, function() {
              var args;
              args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
              return request.callback.apply(request, args);
            });
          }
        });
      } else {
        return router.run(request, function() {
          var args;
          args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
          return request.callback.apply(request, args);
        });
      }
    });
    pubsubBackend.on('notificationPush', function(opts) {
      logger.trace("notificationPush: " + (inspect(opts)));
      opts.operation = 'push-inbox';
      return router.run(opts, function() {});
    });
    pubsubBackend.on('syncNeeded', function(server) {
      return router.syncServer(server, function() {});
    });
    pubsubBackend.on('authorizationPrompt', function(opts) {
      return pubsubBackend.authorizeFor(opts.sender, opts.nodeOwner, function(err, valid) {
        if (valid) {
          opts.type = 'authorizationPrompt';
          return router.notify(opts);
        }
      });
    });
    pubsubBackend.on('authorizationConfirm', function(opts) {
      opts.operation = 'confirm-subscriber-authorization';
      return router.run(opts, function() {});
    });
    xmppConn.on('userOffline', function(user) {
      return router.onUserOffline(user);
    });
    xmppConn.on('online', function() {
      var saidHello, sync;
      logger.info("XMPP connection established");
      process.title = "buddycloud-server " + version + ": " + xmppConn.jid;
      saidHello = false;
      model.forListeners(function(listener) {
        if (!saidHello) {
          logger.info("server successfully started");
          saidHello = true;
        }
        return xmppConn.probePresence(listener);
      });
      sync = function() {
        return router.setupSync(Math.ceil((config.modelConfig.poolSize || 2) / 2));
      };
      return setTimeout(sync, 5000);
    });
    if (!(config.advertiseComponents != null)) {
      config.advertiseComponents = [];
    }
    _results = [];
    for (index in config.advertiseComponents) {
      componentConfig = {};
      _ref1 = config.xmpp;
      for (key in _ref1) {
        value = _ref1[key];
        componentConfig[key] = value;
      }
      componentConfig.jid = config.advertiseComponents[index];
      componentConfig.reconnect = true;
      connection = new xmpp.Component(componentConfig);
      connection.on("error", function(e) {
        return logger.error(e);
      });
      _results.push(connection.on("stanza", function(stanza) {
        var child, from, i, query, reply, _ref2;
        logger.trace("<< Extra connection request: " + (stanza.toString()));
        from = stanza.attrs.from;
        if (stanza.name === 'iq' && stanza.attrs.type === 'get') {
          if (!(stanza.children != null)) {
            stanza.children = [];
          }
          _ref2 = stanza.children;
          for (i in _ref2) {
            child = _ref2[i];
            if (child.name === 'query') {
              query = child;
            }
          }
          if (!(query != null)) {
            return;
          }
          switch (query.attrs.xmlns) {
            case NS.DISCO_ITEMS:
              reply = new xmpp.Element("iq", {
                from: stanza.attrs.to,
                to: stanza.attrs.from,
                id: stanza.attrs.id || "",
                type: "result",
                xmlns: Connection.NS_STREAM
              }).c('query', {
                xmlns: NS.DISCO_ITEMS
              }).c('item', {
                jid: config.xmpp.jid,
                name: 'buddycloud-server'
              });
              break;
            case NS.DISCO_INFO:
              reply = new xmpp.Element("iq", {
                from: stanza.attrs.to,
                to: stanza.attrs.from,
                id: stanza.attrs.id || "",
                type: "result",
                xmlns: Connection.NS_STREAM
              }).c('query', {
                xmlns: NS.DISCO_INFO
              }).c('feature', {
                "var": NS.DISCO_INFO
              }).up().c('feature', {
                "var": NS.DISCO_ITEMS
              }).up().c('feature', {
                "var": NS.REGISTER
              }).up().c('identity', {
                category: 'pubsub',
                type: 'service',
                name: 'Buddycloud proxy domain'
              });
              break;
            default:
              return;
          }
          logger.trace("<< Extra connection response: " + (reply.root().toString()));
          return connection.send(reply);
        }
      }));
    }
    return _results;
  });

}).call(this);
