// Generated by CoffeeScript 1.3.3
(function() {
  var NS_AS, NS_ATOM, NS_THR, errors, logger, nodeRegexp, normalizeActivityStream, normalizeAuthor, normalizeEntry, normalizeId, normalizeLink, normalizePublished, normalizeTextNodes, normalizeUpdated;

  logger = require('./logger').makeLogger('normalize');

  errors = require('./errors');

  NS_ATOM = "http://www.w3.org/2005/Atom";

  NS_AS = "http://activitystrea.ms/spec/1.0/";

  NS_THR = "http://purl.org/syndication/thread/1.0";

  nodeRegexp = /^\/user\/([^\/]+)\/(.+)/;

  exports.normalizeItem = function(req, oldItem, item, cb) {
    var m, nodeType, req2, _ref;
    if (!(m = nodeRegexp.exec(req.node))) {
      return cb(new errors.BadRequest("No recognized node type"));
    }
    nodeType = m[2];
    if (nodeType === 'posts' || nodeType === 'status') {
      if ((_ref = item.el) != null ? _ref.is('entry', NS_ATOM) : void 0) {
        req2 = Object.create(req);
        req2.nodeType = nodeType;
        req2.oldItem = oldItem;
        req2.item = item;
        return normalizeEntry(req2, function(err, req3) {
          return cb(err, req3 != null ? req3.item : void 0);
        });
      } else {
        return cb(new errors.BadRequest("Item payload must be an ATOM entry"));
      }
    } else {
      return cb(null, item);
    }
  };

  normalizeEntry = function(req, cb) {
    try {
      normalizeTextNodes(req);
      normalizeAuthor(req);
      normalizeId(req);
      normalizePublished(req);
      normalizeUpdated(req);
      normalizeLink(req);
      normalizeActivityStream(req);
      return cb(null, req);
    } catch (e) {
      logger.error(e.stack);
      return cb(e);
    }
  };

  normalizeTextNodes = function(req) {
    var deleteEmptyTextNodes;
    deleteEmptyTextNodes = function(el) {
      var child, cleanChildren, _i, _len, _ref;
      if (!el.is('content', NS_ATOM)) {
        cleanChildren = [];
        _ref = el.children;
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          child = _ref[_i];
          if (typeof child === 'string') {
            if (child.trim().length !== 0) {
              cleanChildren.push(child);
            }
          } else {
            cleanChildren.push(deleteEmptyTextNodes(child));
          }
        }
        el.children = cleanChildren;
      }
      return el;
    };
    return req.item.el = deleteEmptyTextNodes(req.item.el);
  };

  normalizeAuthor = function(req) {
    var authorEls;
    authorEls = req.item.el.getChildren('author');
    if (authorEls.length < 1) {
      authorEls = [req.item.el.c('author')];
    }
    return authorEls.forEach(function(authorEl) {
      authorEl.remove('uri');
      return authorEl.c('uri').t("acct:" + req.actor);
    });
  };

  normalizeId = function(req) {
    req.item.el.remove("id", NS_ATOM);
    return req.item.el.c("id").t(req.item.id);
  };

  normalizePublished = function(req) {
    var published;
    req.item.el.remove("published", NS_ATOM);
    published = new Date().toISOString();
    if (req.oldItem != null) {
      req.oldItem.getChildren("published").forEach(function(publishedEl) {
        return published = publishedEl.getText();
      });
    }
    return req.item.el.c("published").t(published);
  };

  normalizeUpdated = function(req) {
    var updated;
    req.item.el.remove("updated", NS_ATOM);
    updated = new Date().toISOString();
    return req.item.el.c("updated").t(updated);
  };

  normalizeActivityStream = function(req) {
    var irtEl, objectType, verb;
    irtEl = req.item.el.getChild('in-reply-to', NS_THR);
    if (!req.item.el.getChild('verb', NS_AS)) {
      verb = irtEl ? 'comment' : 'post';
      req.item.el.c('verb', {
        xmlns: NS_AS
      }).t(verb);
    }
    if (!req.item.el.getChild('object', NS_AS)) {
      objectType = irtEl ? 'comment' : 'note';
      return req.item.el.c('object', {
        xmlns: NS_AS
      }).c('object-type').t(objectType);
    }
  };

  normalizeLink = function(req) {
    var alreadyPresent, link;
    link = ("xmpp:" + req.me + "?pubsub;action=retrieve;") + ("node=" + (encodeURI(req.node)) + ";") + ("item=" + (encodeURI(req.item.id)));
    alreadyPresent = req.item.el.children.some(function(child) {
      return typeof child !== 'string' && child.is('link') && child.attrs.rel === 'self' && child.attrs.href === link;
    });
    if (!alreadyPresent) {
      return req.item.el.c('link', {
        rel: 'self',
        href: link
      });
    }
  };

}).call(this);
